# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 09.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 10, task 5.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name
# pylint: disable=import-error

import os

import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage as ndi
from skimage import morphology, feature

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num


def create_disk_image():
  """Create a binary image containing two overlapping disks"""
  center1 = -10
  center2 = -center1
  dist = np.sqrt(2*(2*center1)**2)
  radius = 1.4*dist/2
  lims = (np.floor(center1 - 1.2*radius), np.ceil(center2 + 1.2*radius))
  x, y = np.meshgrid(np.arange(lims[0], lims[1]), np.arange(lims[0], lims[1]))
  disk1 = np.sqrt((x - center1)**2 + (y - center1)**2) <= radius
  disk2 = np.sqrt((x - center2)**2 + (y - center2)**2) <= radius
  disks = np.logical_or(disk1, disk2)
  return disks


def main():
  """Binary segmentation"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 10')
  print('Task 5')
  print('-'*80)

  # Configurations
  result_dir = 'results/images'
  fig_num = 0

  # Create image with two overlapping disks
  disk_img = create_disk_image()
  fig_num = plot_image(disk_img, fig_num,
                       write_file=os.path.join(result_dir, 't05_disk_image.png'))

  # Compute the distance_transform (a function that takes a binary array as
  # input, and outputs an array where each element (i,j) has the eclidian
  # distance to the nearest background (0) value in the input array as value.
  dist_trans_img = ndi.distance_transform_edt(disk_img)
  fig_num = plot_image(dist_trans_img, fig_num,
                       write_file=os.path.join(result_dir, 't05_dist_trans_image.png'))

  # Create markers (seeds for the watershed) at the local peaks
  local_maxima = feature.peak_local_max(dist_trans_img, indices=False,
                                        footprint=np.ones((3, 3)), labels=disk_img)
  markers = ndi.label(local_maxima)[0]

  # Complement the distance tranform
  neg_dist_trans_img = -dist_trans_img.astype('float64')
  #neg_dist_trans_img[disk_img == 0] = -float('Inf')
  fig_num = plot_image(neg_dist_trans_img, fig_num,
                       write_file=os.path.join(result_dir, 't05_neg_dist_trans_image.png'))

  # Compute watershed transform
  labels = morphology.watershed(neg_dist_trans_img, markers, mask=disk_img)
  fig_num = plot_image(labels, fig_num,
                       write_file=os.path.join(result_dir, 't05_label_image.png'))

  plt.show()

if __name__ == "__main__":
  main()
